package org.eclipse.ui.internal;



import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;

import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;



public class PartLocator {
	
	private static boolean fgEnabled= true;
	private static List fgExceptions= new ArrayList();
	
	private PartLocator() {
	}
	
	
	//----------------- managing behavior ----------------
	
	public static void enable(boolean enable) {
		fgEnabled= enable;
	}
	
	public static void excludePerspectiv(String perspectiveID, boolean exclude) {
		if (!exclude)
			fgExceptions.remove(perspectiveID);
		else if ( !fgExceptions.contains(perspectiveID))
			fgExceptions.add(perspectiveID);
	}
	
	private static boolean includePage(IWorkbenchPage page) {
		String pespectiveID= page.getPerspective().getId();
		return !fgExceptions.contains(pespectiveID);
	}
	
	//------------------- open view part --------------------
	
	private static IWorkbenchPage internalFindView(IWorkbenchWindow window, IWorkbenchPage inputProvider, String viewId) {
		
		IAdaptable input= inputProvider.getInput();
		IWorkbenchPage[] pages= window.getPages();
		
		for (int i= 0; i < pages.length; i++) {
			
			if (pages[i] == inputProvider || !includePage(pages[i]))
				continue;
			
			IAdaptable pageInput= pages[i].getInput();
			if (input != null && input.equals(pageInput) || input == null && pageInput == null) {
				IViewPart part= pages[i].findView(viewId);
				if (part != null)
					return pages[i];
			}
		}
		
		return null;
	}	
	
	private static IWorkbenchPage findView(IWorkbenchPage requester, String viewId) {
				
		IViewPart part= requester.findView(viewId);
		if (part != null)
			return requester;
			
		IWorkbenchWindow window= requester.getWorkbenchWindow();
		IWorkbenchPage page= internalFindView(window, requester, viewId);
		if (page != null)
			return page;
		
		IWorkbench workbench=window.getWorkbench();
		IWorkbenchWindow[] windows= workbench.getWorkbenchWindows();
		
		for (int i= 0; i < windows.length; i++) {
			
			if (windows[i] == window)
				continue;
			
			page= internalFindView(windows[i], requester, viewId);
			if (page != null)
				return page;
		}
		
		return null;
	}
	
	public static IViewPart showView(IWorkbenchPage requester, String viewId) throws PartInitException {
		
		if ( !(fgEnabled || includePage(requester)))
			return null;
			
		IWorkbenchPage page= findView(requester, viewId);
		if (page != null && page != requester) {
			IWorkbenchWindow window= page.getWorkbenchWindow();
			if (window != null) {
				window.setActivePage(page);
				window.getShell().open();
			}
			return page.showView(viewId);
		}
		
		if (page == requester)
			return page.findView(viewId);
			
		return null;	
	}
	
	
	
	//------------------- open editor part --------------------------------
	
	
	private static IEditorPart internalFindEditor(IWorkbenchPage page, IEditorInput input) {
		IEditorPart[] editors= page.getEditors();
		for (int i= 0; i < editors.length; i++) {
			if (input.equals(editors[i].getEditorInput()))
				return editors[i];
		}
		return null;
	}
	
	private static IWorkbenchPage internalFindEditor(IWorkbenchWindow window, IWorkbenchPage inputProvider, IEditorInput editorInput) {
		
		IAdaptable input= inputProvider.getInput();
		IWorkbenchPage[] pages= window.getPages();
		
		for (int i= 0; i < pages.length; i++) {
			
			if (pages[i] == inputProvider || !includePage(pages[i]))
				continue;
			
			IAdaptable pageInput= pages[i].getInput();
			if (input != null && input.equals(pageInput) || input == null && pageInput == null) {
				IEditorPart part= internalFindEditor(pages[i], editorInput);
				if (part != null)
					return pages[i];
			}
		}
		
		return null;
	}
	
	private static IWorkbenchPage findEditor(IWorkbenchPage requester, IEditorInput input) {
		
		IEditorPart editor= internalFindEditor(requester, input);
		if (editor != null)
			return requester;
			
		IWorkbenchWindow window= requester.getWorkbenchWindow();
		IWorkbenchPage page= internalFindEditor(window, requester, input);
		if (page != null)
			return page;
		
		IWorkbench workbench=window.getWorkbench();
		IWorkbenchWindow[] windows= workbench.getWorkbenchWindows();
		
		for (int i= 0; i < windows.length; i++) {
			
			if (windows[i] == window)
				continue;
			
			page= internalFindEditor(windows[i], requester, input);
			if (page != null)
				return page;
		}
		
		return null;
	}
	
	private static IWorkbenchPage internalFindEditorPage(IWorkbenchWindow window, IWorkbenchPage skip) {
		
		IWorkbenchPage[] pages= window.getPages();
		
		for (int i= 0; i < pages.length; i++) {
			
			if (pages[i] == skip || !includePage(pages[i]))
				continue;
			
			if (pages[i].isEditorAreaVisible())
				return pages[i];
		}
		
		return null;
	}
	
	private static IWorkbenchPage findEditorPage(IWorkbenchPage requester) {
		
		if (requester.isEditorAreaVisible())
			return requester;
			
		IWorkbenchWindow window= requester.getWorkbenchWindow();
		IWorkbenchPage page= internalFindEditorPage(window, requester);
		if (page != null)
			return page;
		
		IWorkbench workbench=window.getWorkbench();
		IWorkbenchWindow[] windows= workbench.getWorkbenchWindows();
		
		for (int i= 0; i < windows.length; i++) {
			
			if (windows[i] == window)
				continue;
			
			page= internalFindEditorPage(windows[i], null);
			if (page != null)
				return page;
		}
		
		return null;
	}
	
	public static IEditorPart openEditor(IWorkbenchPage requester, FileEditorInput input, String editorId) throws PartInitException {
		
		if ( !(fgEnabled || includePage(requester)))
			return null;
			
		IWorkbenchPage page= findEditor(requester, input);
		if (page != null && page != requester)
			return (editorId != null ? page.openEditor(input, editorId) : page.openEditor(input.getFile()));
			
		if (page == requester)
			return internalFindEditor(requester, input);
			
		page= findEditorPage(requester);
		if (page != requester && page != null) {
			
			IWorkbenchWindow window= page.getWorkbenchWindow();
			if (window != null) {
				window.setActivePage(page);
				window.getShell().open();
			}
			
			
			return (editorId != null ? page.openEditor(input, editorId) : page.openEditor(input.getFile()));
		}
		
		return null;
	}
	
	public static IEditorPart openEditor(IWorkbenchPage requester, IEditorInput input, String editorId, boolean activate) throws PartInitException {
		
		if ( !(fgEnabled || includePage(requester)))
			return null;
			
		IWorkbenchPage page= findEditor(requester, input);
		if (page != null && page != requester)
			return page.openEditor(input, editorId, activate);
			
		if (page == requester)
			return internalFindEditor(requester, input);
			
		page= findEditorPage(requester);
		if (page != requester && page != null) {
			
			IWorkbenchWindow window= page.getWorkbenchWindow();
			if (window != null) {
				window.setActivePage(page);
				window.getShell().open();
			}
			
			
			return page.openEditor(input, editorId, activate);
		}
		
		return null;
	}	
}

